@@ -4,11 +4,9 @@ use std::ffi::OsStr;
 use std::io::Write;
 use std::path::Path;
 use std::process::exit;
-use std::str::FromStr;
 
 use boolinator::Boolinator;
 use chrono::prelude::Local;
-use semver::Identifier;
 use structopt::StructOpt;
 
 use crate::error::FatalError;
@@ -23,6 +21,8 @@ mod replace;
 mod shell;
 mod version;
 
+use version::VersionExt;
+
 static NOW: once_cell::sync::Lazy<String> =
     once_cell::sync::Lazy::new(|| Local::now().format("%Y-%m-%d").to_string());
 
@@ -48,12 +48,17 @@ fn exclude_paths<'m>(
 ) -> Vec<&'m Path> {
     let base_path = pkg_meta
         .manifest_path
+        .as_std_path()
         .parent()
         .unwrap_or_else(|| Path::new("/"));
     ws_pkgs
         .iter()
         .filter_map(|p| {
-            let cur_path = p.manifest_path.parent().unwrap_or_else(|| Path::new("/"));
+            let cur_path = p
+                .manifest_path
+                .as_std_path()
+                .parent()
+                .unwrap_or_else(|| Path::new("/"));
             if cur_path != base_path && cur_path.starts_with(base_path) {
                 Some(cur_path)
             } else {
@@ -92,6 +97,12 @@ struct Version {
     version_string: String,
 }
 
+impl Version {
+    fn is_prerelease(&self) -> bool {
+        self.version.is_prerelease()
+    }
+}
+
 struct Dependency<'m> {
     pkg: &'m cargo_metadata::Package,
     req: &'m semver::VersionReq,
@@ -105,14 +116,15 @@ impl<'m> PackageRelease<'m> {
         ws_pkgs: &[&'m cargo_metadata::Package],
         pkg_meta: &'m cargo_metadata::Package,
     ) -> Result<Option<Self>, error::FatalError> {
-        let manifest_path = pkg_meta.manifest_path.as_path();
+        let manifest_path = pkg_meta.manifest_path.as_std_path();
         let cwd = manifest_path.parent().unwrap_or_else(|| Path::new("."));
 
         let config = {
             let mut release_config = config::Config::default();
 
             if !args.isolated {
-                let cfg = config::resolve_config(&ws_meta.workspace_root, manifest_path)?;
+                let cfg =
+                    config::resolve_config(ws_meta.workspace_root.as_std_path(), manifest_path)?;
                 release_config.update(&cfg);
             }
 
@@ -178,41 +190,13 @@ impl<'m> PackageRelease<'m> {
             template.render(config.tag_name())
         };
 
-        let mut is_pre_release = false;
-        let version = {
-            let mut potential_version = prev_version.version.clone();
-            if let Ok(bump_level) = version::BumpLevel::from_str(&args.level_or_version) {
-                // bump level
-                if bump_level.bump_version(&mut potential_version, args.metadata.as_ref())? {
-                    let version = potential_version;
-                    let version_string = version.to_string();
-                    is_pre_release = bump_level.is_pre_release();
-                    Some(Version {
-                        version,
-                        version_string,
-                    })
-                } else {
-                    None
-                }
-            } else {
-                // given version
-                match semver::Version::parse(&args.level_or_version)? {
-                    version if version > potential_version => {
-                        is_pre_release = version.is_prerelease();
-                        Some(Version {
-                            version,
-                            version_string: args.level_or_version.to_owned(),
-                        })
-                    }
-                    version if version == potential_version => None,
-                    _ => {
-                        return Err(error::FatalError::UnsupportedVersionReq(
-                            "Cannot release version smaller than current one".to_owned(),
-                        ));
-                    }
-                }
-            }
-        };
+        let version = args
+            .level_or_version
+            .bump(&prev_version.version, args.metadata.as_deref())?;
+        let is_pre_release = version
+            .as_ref()
+            .map(Version::is_prerelease)
+            .unwrap_or(false);
         let dependents = if version.is_some() {
             find_dependents(ws_meta, pkg_meta)
                 .map(|(pkg, dep)| Dependency { pkg, req: &dep.req })
@@ -242,9 +226,7 @@ impl<'m> PackageRelease<'m> {
         let post_version = if !is_pre_release && !config.no_dev_version() {
             let mut post = base.version.clone();
             post.increment_patch();
-            post.pre.push(Identifier::AlphaNumeric(
-                config.dev_version_ext().to_owned(),
-            ));
+            post.pre = semver::Prerelease::new(config.dev_version_ext())?;
             let post_string = post.to_string();
 
             Some(Version {
@@ -334,7 +316,7 @@ fn update_dependent_versions(
                         );
                         if !dry_run {
                             cargo::set_dependency_version(
-                                &dep.pkg.manifest_path,
+                                dep.pkg.manifest_path.as_std_path(),
                                 &pkg.meta.name,
                                 &new_req,
                             )?;
@@ -354,7 +336,7 @@ fn update_dependent_versions(
                     );
                     if !dry_run {
                         cargo::set_dependency_version(
-                            &dep.pkg.manifest_path,
+                            dep.pkg.manifest_path.as_std_path(),
                             &pkg.meta.name,
                             &new_req,
                         )?;
@@ -376,7 +358,7 @@ fn release_workspace(args: &ReleaseOpt) -> Result<i32, error::FatalError> {
         let mut release_config = config::Config::default();
 
         if !args.isolated {
-            let cfg = config::resolve_workspace_config(&ws_meta.workspace_root)?;
+            let cfg = config::resolve_workspace_config(ws_meta.workspace_root.as_std_path())?;
             release_config.update(&cfg);
         }
 
@@ -402,7 +384,7 @@ fn release_workspace(args: &ReleaseOpt) -> Result<i32, error::FatalError> {
     all_pkgs.extend(excluded_pkgs);
     let all_pkgs = all_pkgs;
 
-    let root = git::top_level(&ws_meta.workspace_root)?;
+    let root = git::top_level(ws_meta.workspace_root.as_std_path())?;
     let pkg_releases: Result<HashMap<_, _>, _> = selected_pkgs
         .iter()
         .filter_map(|p| PackageRelease::load(args, &root, &ws_meta, &all_pkgs, p).transpose())
@@ -476,7 +458,7 @@ fn release_packages<'m>(
     git::git_version()?;
     let mut dirty = false;
     if ws_config.consolidate_commits() {
-        if git::is_dirty(&ws_meta.workspace_root)? {
+        if git::is_dirty(ws_meta.workspace_root.as_std_path())? {
             log::warn!("Uncommitted changes detected, please commit before release.");
             dirty = true;
         }
@@ -563,12 +545,12 @@ fn release_packages<'m>(
     }
 
     let git_remote = ws_config.push_remote();
-    let branch = git::current_branch(&ws_meta.workspace_root)?;
+    let branch = git::current_branch(ws_meta.workspace_root.as_std_path())?;
     if branch == "HEAD" {
         log::warn!("Releasing from a detached HEAD");
     }
-    git::fetch(&ws_meta.workspace_root, git_remote, &branch)?;
-    if git::is_behind_remote(&ws_meta.workspace_root, git_remote, &branch)? {
+    git::fetch(ws_meta.workspace_root.as_std_path(), git_remote, &branch)?;
+    if git::is_behind_remote(ws_meta.workspace_root.as_std_path(), git_remote, &branch)? {
         log::warn!("{} is behind {}/{}", branch, git_remote, branch);
     }
 
@@ -688,7 +670,7 @@ fn release_packages<'m>(
             template.render(ws_config.pre_release_commit_message())
         };
         if !git::commit_all(
-            &ws_meta.workspace_root,
+            ws_meta.workspace_root.as_std_path(),
             &shared_commit_msg,
             ws_config.sign_commit(),
             dry_run,
@@ -830,7 +812,7 @@ fn release_packages<'m>(
             template.render(ws_config.post_release_commit_message())
         };
         if !git::commit_all(
-            &ws_meta.workspace_root,
+            ws_meta.workspace_root.as_std_path(),
             &shared_commit_msg,
             ws_config.sign_commit(),
             dry_run,
@@ -868,7 +850,7 @@ fn release_packages<'m>(
         if shared_push {
             log::info!("Pushing HEAD to {}", git_remote);
             if !git::push(
-                &ws_meta.workspace_root,
+                ws_meta.workspace_root.as_std_path(),
                 git_remote,
                 ws_config.push_options(),
                 dry_run,
@@ -900,8 +882,8 @@ struct ReleaseOpt {
     workspace: clap_cargo::Workspace,
 
     /// Release level or version: bumping specified version field or remove prerelease extensions by default. Possible level value: major, minor, patch, release, rc, beta, alpha or any valid semver version that is greater than current version
-    #[structopt(case_insensitive(true), default_value = "release")]
-    level_or_version: String,
+    #[structopt(default_value)]
+    level_or_version: TargetVersion,
 
     #[structopt(short = "m")]
     /// Semver metadata
@@ -1117,6 +1099,81 @@ enum Command {
     Release(ReleaseOpt),
 }
 
+#[derive(Clone, Debug)]
+enum TargetVersion {
+    Relative(version::BumpLevel),
+    Absolute(semver::Version),
+}
+
+impl TargetVersion {
+    fn bump(
+        &self,
+        current: &semver::Version,
+        metadata: Option<&str>,
+    ) -> Result<Option<Version>, FatalError> {
+        match self {
+            TargetVersion::Relative(bump_level) => {
+                let mut potential_version = current.to_owned();
+                if bump_level.bump_version(&mut potential_version, metadata)? {
+                    let version = potential_version;
+                    let version_string = version.to_string();
+                    Ok(Some(Version {
+                        version,
+                        version_string,
+                    }))
+                } else {
+                    Ok(None)
+                }
+            }
+            TargetVersion::Absolute(version) => {
+                if current < version {
+                    Ok(Some(Version {
+                        version: version.to_owned(),
+                        version_string: version.to_string(),
+                    }))
+                } else if current == version {
+                    Ok(None)
+                } else {
+                    return Err(error::FatalError::UnsupportedVersionReq(
+                        "Cannot release version smaller than current one".to_owned(),
+                    ));
+                }
+            }
+        }
+    }
+}
+
+impl Default for TargetVersion {
+    fn default() -> Self {
+        TargetVersion::Relative(version::BumpLevel::Release)
+    }
+}
+
+impl std::fmt::Display for TargetVersion {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
+        match self {
+            TargetVersion::Relative(bump_level) => {
+                write!(f, "{}", bump_level)
+            }
+            TargetVersion::Absolute(version) => {
+                write!(f, "{}", version)
+            }
+        }
+    }
+}
+
+impl std::str::FromStr for TargetVersion {
+    type Err = FatalError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if let Ok(bump_level) = version::BumpLevel::from_str(s) {
+            Ok(TargetVersion::Relative(bump_level))
+        } else {
+            Ok(TargetVersion::Absolute(semver::Version::parse(s)?))
+        }
+    }
+}
+
 pub fn get_logging(level: log::Level) -> env_logger::Builder {
     let mut builder = env_logger::Builder::new();
 
